#
#  AutomaticCrashlogCleanup E2
#
#  $Id$
#
#  Coded by Shaderman (c) 2010
#  Support: www.dreambox.tools
#
#  This plugin is licensed under the Creative Commons 
#  Attribution-NonCommercial-ShareAlike 3.0 Unported 
#  License. To view a copy of this license, visit
#  http://creativecommons.org/licenses/by-nc-sa/3.0/ or send a letter to Creative
#  Commons, 559 Nathan Abbott Way, Stanford, California 94305, USA.
#
#  Alternatively, this plugin may be distributed and executed on hardware which
#  is licensed by Dream Multimedia GmbH.

#  This plugin is NOT free software. It is open source, you are allowed to
#  modify it (if you keep the license), but it may not be commercially 
#  distributed other than under the conditions noted above.
#

from Plugins.Plugin import PluginDescriptor
from Screens.Screen import Screen
from Components.ActionMap import ActionMap
from Components.Sources.StaticText import StaticText
from Components.config import config, ConfigSubsection, ConfigSelection, getConfigListEntry
from Components.ConfigList import ConfigListScreen
from time import time
from glob import glob
from os import remove as os_remove

# for localized messages
from . import _

config.plugins.AutomaticCrashlogCleanup = ConfigSubsection()
config.plugins.AutomaticCrashlogCleanup.keep = ConfigSelection(default = "-1", choices = [("-1",_("all")), ("10",_("last 10")),("5",_("last 5")),("3",_("last 3")),("1",_("last 1"))])
config.plugins.AutomaticCrashlogCleanup.deleteOlderThan = ConfigSelection(default = "-1", choices = [("-1",_("disabled")), ("28",_("4 weeks")),("14",_("2 weeks")), ("7",_("1 week")), ("3",_("3 days")),("1",_("1 day"))])

class AutomaticCrashlogCleanupSetup(Screen, ConfigListScreen): # config

	skin = """
		<screen position="center,center" size="560,400" title="%s" >
			<ePixmap pixmap="skin_default/buttons/red.png" position="0,0" zPosition="0" size="140,40" transparent="1" alphatest="on" />
			<ePixmap pixmap="skin_default/buttons/green.png" position="140,0" zPosition="0" size="140,40" transparent="1" alphatest="on" />
			<ePixmap pixmap="skin_default/buttons/yellow.png" position="280,0" zPosition="0" size="140,40" transparent="1" alphatest="on" />
			<ePixmap pixmap="skin_default/buttons/blue.png" position="420,0" zPosition="0" size="140,40" transparent="1" alphatest="on" />
			<widget render="Label" source="key_red" position="0,0" size="140,40" zPosition="5" valign="center" halign="center" backgroundColor="red" font="Regular;21" transparent="1" foregroundColor="white" shadowColor="black" shadowOffset="-1,-1" />
			<widget render="Label" source="key_green" position="140,0" size="140,40" zPosition="5" valign="center" halign="center" backgroundColor="red" font="Regular;21" transparent="1" foregroundColor="white" shadowColor="black" shadowOffset="-1,-1" />
			<widget name="config" position="20,50" size="520,330" scrollbarMode="showOnDemand" />
		</screen>""" % _("Automatic Crashlog Cleanup Setup")

	def __init__(self, session):
		Screen.__init__(self, session)
		self["key_red"] = StaticText(_("Cancel"))
		self["key_green"] = StaticText(_("OK"))
		self.list = [ ]
		self.list.append(getConfigListEntry(_("Maximum number of crashlogs to keep"), config.plugins.AutomaticCrashlogCleanup.keep))
		self.list.append(getConfigListEntry(_("Always delete crashlogs older than"), config.plugins.AutomaticCrashlogCleanup.deleteOlderThan))
		ConfigListScreen.__init__(self, self.list, session)
		self["setupActions"] = ActionMap(["SetupActions", "ColorActions"],
		{
			"green": self.keySave,
			"cancel": self.keyClose,
		}, -2)

	def keySave(self):
		for x in self["config"].list:
			x[1].save()
		self.close(True)

	def keyClose(self):
		for x in self["config"].list:
			x[1].cancel()
		self.close(False)

class AutomaticCrashlogCleanup:
	def __init__(self, session):
		self.session = session
		print "[AutomaticCrashlogCleanup] Starting AutomaticCrashlogCleanup..."
		self.cleanupCrashlogs() # always check immediately after starting plugin

	def cleanupCrashlogs(self):
		if int(config.plugins.AutomaticCrashlogCleanup.keep.value) > -1 or int(config.plugins.AutomaticCrashlogCleanup.deleteOlderThan.value) > -1: # check only if feature is enabled
			print "[AutomaticCrashlogCleanup] Cleaning up crashlogs"
			self.crashlogList = glob('/media/hdd/enigma2_crash_*.log')
			self.crashlogList.sort()
			self.numCrashlogs = len(self.crashlogList)
			if self.numCrashlogs == 0:
				print "[AutomaticCrashlogCleanup] No crashlogs found!"
			else:
				self.filterCrashlogs()
		else:
			print "[AutomaticCrashlogCleanup] disabled"

	def filterCrashlogs(self):
		self.deleteList = [ ]
		
		keep = int(config.plugins.AutomaticCrashlogCleanup.keep.value)
		if keep > -1: # don't keep all crashlogs
			if keep >= self.numCrashlogs:
				print "[AutomaticCrashlogCleanup] Found %i crashlog(s), keeping max %i" % (self.numCrashlogs, keep)
			else:
				print "[AutomaticCrashlogCleanup] Keeping the %i latest crashlogs" % keep
				# add all crashlogs > config.plugins.AutomaticCrashlogCleanup.keep.value
				# to a new list. the crashlogs in this new list will be deleted later.
				self.deleteList = self.crashlogList[0 : self.numCrashlogs - keep]

		if int(config.plugins.AutomaticCrashlogCleanup.deleteOlderThan.value) > -1:
			print "[AutomaticCrashlogCleanup] Searching for outdated crashlogs"
			now = int(time())
			# 86400 = one day in seconds
			deleteOlderThan = now - 86400 * int(config.plugins.AutomaticCrashlogCleanup.deleteOlderThan.value)
			
			if keep > -1: # don't keep all crashlogs
				# start checking the range in self.crashlogList which wasn't copied to
				# self.deleteList
				i = self.numCrashlogs - keep
				# if there are less crashlogs than we want to keep, check the
				# whole crashlog list
				if i < 0:
					i = 0
			else:
				i = 0
			
			while i < self.numCrashlogs:
				crashTime = self.crashlogList[i].lstrip('/media/hdd/enigma2_crash_')
				crashTime = crashTime.rstrip('.log')
				if int(crashTime) > deleteOlderThan:
					break
				self.deleteList.append(self.crashlogList[i])
				i += 1
			
			print "[AutomaticCrashlogCleanup] Found %i outdated crashlog(s)" % i

		for crashlog in self.deleteList:
			os_remove(crashlog)

		print "[AutomaticCrashlogCleanup] Deleted %i crashlog(s)" % len(self.deleteList)

def autostart(session, **kwargs):
	AutomaticCrashlogCleanup(session) # start plugin at sessionstart
	
def setup(session, **kwargs):
	session.open(AutomaticCrashlogCleanupSetup) # start setup

def startSetup(menuid):
	if menuid != "merlin": # show setup only in merlin level menu
		return []
	return [(_("Automatic Crashlog Cleanup Setup"), setup, "AutomaticCrashlogCleanup", 46)]
	
def Plugins(**kwargs):
	return [PluginDescriptor(where = [PluginDescriptor.WHERE_SESSIONSTART], fnc = autostart), PluginDescriptor(name="Automatic Crashlog Cleanup Setup", description=_("Automatic Crashlog Cleanup Setup"), where = PluginDescriptor.WHERE_MENU, fnc=startSetup) ]

